//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_FOLLOWERVEHICLE_H
#define PROTOCOL_FOLLOWERVEHICLE_H


#include <mutex>
#include "NetworkingLib/DatagramReceiver.h"
#include "NetworkingLib/DatagramSender.h"
#include "Vehicle.h"
#include "PlatoonMessage.h"
#include "Utils.h"

namespace platoonProtocol
{

class FollowerVehicle : public Vehicle
{
private:
    struct PrivateTag
    {
    };

public:
    using Ptr = std::shared_ptr<FollowerVehicle>;

    static Ptr create(networking::Networking & net, const NetworkInfo & info);

    // This should not be used outside.
    FollowerVehicle(PrivateTag, networking::Networking & net, const NetworkInfo & info);

    void stop() override;

    PlatoonConfig getPlatoonConfig() const noexcept;

    void setOnPlatoonConfigUpdatedCallback(Callback callback)
    { onPlatoonConfigUpdatedCallback = callback; }

    void addVehicleToRequest(VehicleId vehicleId);

protected:
    void doCreatePlatoon() override;

    void doRunPlatoon() override;

    void doLeavePlatoon() override;

    void receiveMessage(const PlatoonMessage & message) override;

    Ptr shared_from_this();

private:
    static constexpr networking::time::Duration HEARTBEAT_INTERVAL = std::chrono::milliseconds(50);
    static constexpr networking::time::Duration BROADCAST_TIMEOUT = std::chrono::milliseconds(500);

    VehicleId leader;
    std::vector<VehicleId> vehiclesToRequest;
    std::vector<VehicleId>::iterator vehiclesToRequestIter;
    bool waitingForResponse{false};
    networking::time::TimePoint lastBroadcastTimestamp;
    networking::time::Timer::Ptr heartbeatTimer;
    Callback onPlatoonConfigUpdatedCallback;

    void sendPlatoonCreateRequestToNextVehicle();

    void sendPlatoonCreateRequest(VehicleId vehicleId);

    void sendRejectResponse(const PlatoonMessage & request);

    void receiveResponse(const PlatoonMessage & response);

    void startHeartbeat();

    void stopHeartbeat();

    void startBroadcast();

    void receiveBroadcast(const PlatoonMessage & broadcast);

    void sendLeavePlatoonMessage();
};

}

#endif //PROTOCOL_FOLLOWERVEHICLE_H
