//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_UTILS_H
#define PROTOCOL_UTILS_H

#include <iterator>

using boost::asio::ip::tcp;

namespace platoonProtocol
{
namespace utils
{

/**
 * Sets the iterator to the next element.
 * If the iterator is at container.end() it moves to back to container.begin()
 * @tparam ForwardIterator
 * @tparam Container
 * @param iter
 * @param container
 */
template<typename ForwardIterator, typename Container>
void cycle(ForwardIterator & iter, Container & container)
{
    if (std::next(iter) == container.end())
        iter = container.begin();
    else
        iter++;
}

/**
 * @tparam Map
 * @tparam T
 * @param map
 * @return A vector of the keys contained in map
 */
template<template<typename...> class Map, typename Key, typename Value>
std::vector<Key> keys(const Map<Key, Value> & map)
{
    std::vector<Key> mapKeys;
    for (const auto & i : map)
        mapKeys.emplace_back(i.first);
    return mapKeys;
};

/**
 * Safe way to iterate over the elements in a map and delete elements from it
 * which you cannot do with the standard range based for loop.
 * @tparam Map
 * @tparam LoopBody
 * @param map
 * @param body
 */
template<typename Map, typename LoopBody>
void foreachInMap(Map & map, LoopBody body)
{
    for (const auto & key : keys(map))
        body(map[key]);
}

}
}

#endif //PROTOCOL_UTILS_H